==> aptechka-0.42.1.tgz: package/lib/en3/index.js <==
var ue = Object.defineProperty, me = Object.defineProperties;
var fe = Object.getOwnPropertyDescriptors;
var J = Object.getOwnPropertySymbols;
var ge = Object.prototype.hasOwnProperty, Ee = Object.prototype.propertyIsEnumerable;
var ee = (s) => {
  throw TypeError(s);
};
var Q = (s, e, i) => e in s ? ue(s, e, { enumerable: !0, configurable: !0, writable: !0, value: i }) : s[e] = i, j = (s, e) => {
  for (var i in e || (e = {}))
    ge.call(e, i) && Q(s, i, e[i]);
  if (J)
    for (var i of J(e))
      Ee.call(e, i) && Q(s, i, e[i]);
  return s;
}, te = (s, e) => me(s, fe(e));
var ie = (s, e, i) => e.has(s) || ee("Cannot " + i);
var t = (s, e, i) => (ie(s, e, "read from private field"), i ? i.call(s) : e.get(s)), r = (s, e, i) => e.has(s) ? ee("Cannot add the same private member more than once") : e instanceof WeakSet ? e.add(s) : e.set(s, i), h = (s, e, i, n) => (ie(s, e, "write to private field"), n ? n.call(s, i) : e.set(s, i), i);
import { RESIZE_ORDER as be, TICK_ORDER as ye } from "../order/index.js";
import { windowResizer as se } from "../window-resizer/index.js";
import { ticker as re } from "../ticker/index.js";
import { g as ce } from "../dom-CVWzyXPH.js";
import { Mesh as Z, OrthographicCamera as X, PerspectiveCamera as he, Scene as xe, Vector2 as we, Raycaster as pe, REVISION as De, WebGLRenderer as Re, Vector3 as q } from "three";
import { LayoutBox as ne } from "../layout-box/index.js";
function ve(s) {
  const e = (i) => {
    i.dispose();
    for (const n of Object.keys(i)) {
      const a = i[n];
      a && typeof a == "object" && "minFilter" in a && a.dispose();
    }
  };
  s.traverse((i) => {
    if (i instanceof Z) {
      if (i.geometry.dispose(), !Array.isArray(i.material) && i.material.isMaterial)
        e(i.material);
      else if (Array.isArray(i.material))
        for (const n of i.material) e(n);
    }
  });
}
var S, o, V, p, x, P, D, R;
class ae {
  constructor(e, i) {
    r(this, S);
    r(this, o);
    r(this, V);
    r(this, p);
    r(this, x);
    r(this, P);
    r(this, D);
    r(this, R);
    h(this, S, e), h(this, o, (i == null ? void 0 : i.cameraType) === "orthographic" ? new X() : new he()), h(this, V, new xe()), h(this, p, []), h(this, x, (i == null ? void 0 : i.cameraDistance) || 1e3), t(this, o).near = (i == null ? void 0 : i.cameraNear) || 1, t(this, o).far = (i == null ? void 0 : i.cameraFar) || 11e3, h(this, P, (i == null ? void 0 : i.cameraFov) || "auto"), t(this, o).position.z = t(this, x), t(this, o).name = `Cameras.${t(this, S)}`, h(this, D, ce(i == null ? void 0 : i.sizeElement) || document.documentElement), this.beforeRenderCallback = i == null ? void 0 : i.beforeRender, h(this, R, new ne(t(this, D))), t(this, R).onResize(() => {
      this.resize();
    });
  }
  get name() {
    return t(this, S);
  }
  get camera() {
    return t(this, o);
  }
  get scene() {
    return t(this, V);
  }
  get box() {
    return t(this, R);
  }
  get cameraDistance() {
    return t(this, x);
  }
  set cameraDistance(e) {
    h(this, x, e), this.resize();
  }
  get sizeElement() {
    return t(this, D);
  }
  get isClipped() {
    return t(this, D) !== u.containerElement;
  }
  resize() {
    const { width: e, height: i } = t(this, R);
    t(this, o).userData.controlled || (t(this, o).position.z = t(this, x)), t(this, o) instanceof he ? (t(this, o).aspect = e / i, t(this, o).fov = t(this, P) === "auto" ? 2 * Math.atan(i / 2 / t(this, x)) * (180 / Math.PI) : t(this, o).userData.controlled ? t(this, o).fov : t(this, P)) : t(this, o) instanceof X && (t(this, o).left = e / -2, t(this, o).right = e / 2, t(this, o).top = i / 2, t(this, o).bottom = i / -2), t(this, o).updateProjectionMatrix();
  }
  destroy() {
    t(this, p).forEach((e) => {
      e.userData.box.destroy();
    }), t(this, V).clear(), ve(t(this, V)), u.destroyView(this.name), t(this, R).destroy();
  }
  attachToHTMLElement(e, i, n) {
    const a = new ne(e, te(j({}, n), {
      containerElement: t(this, D),
      cartesian: !0,
      scrollStep: !this.isClipped
    }));
    return a.bindObject(i), i.userData.box = a, t(this, p).push(i), i;
  }
  detachFromHTMLElement(e) {
    h(this, p, t(this, p).filter((i) => i === e ? (e.userData.box.destroy(), !1) : !0));
  }
  add(...e) {
    const i = e[0], n = e[1], a = e[2];
    return n && this.attachToHTMLElement(n, i, a), this.scene.add(i), i;
  }
  remove(e, i) {
    this.scene.remove(e), i && this.detachFromHTMLElement(e);
  }
}
S = new WeakMap(), o = new WeakMap(), V = new WeakMap(), p = new WeakMap(), x = new WeakMap(), P = new WeakMap(), D = new WeakMap(), R = new WeakMap();
var A, k, F, v, I;
class ze {
  constructor(e) {
    r(this, A);
    r(this, k);
    r(this, F);
    r(this, v);
    r(this, I);
    h(this, A, e.targetName || void 0), h(this, k, e.eventDispatcher || e.object3D), h(this, F, e.propagation || !1), h(this, v, e.object3D), h(this, I, t(this, A) ? () => t(this, v).getObjectByName(t(this, A)) || t(this, v) : () => t(this, v));
  }
  get object3D() {
    return t(this, v);
  }
  get eventDispatcher() {
    return t(this, k);
  }
  get propagation() {
    return t(this, F);
  }
  get target() {
    return t(this, I).call(this);
  }
  dispatch(e, i) {
    this.eventDispatcher.dispatchEvent(j({
      originalEvent: i,
      type: e
    }, this.intersection));
  }
}
A = new WeakMap(), k = new WeakMap(), F = new WeakMap(), v = new WeakMap(), I = new WeakMap();
var b, d, H, N, W, Y, _;
class Ve {
  constructor() {
    r(this, b, []);
    r(this, d, []);
    r(this, H, new we());
    r(this, N, new pe());
    r(this, W, (e) => {
      for (let i = 0; i < t(this, d).length; i++)
        t(this, d)[i].dispatch("pointerDown", e);
    });
    r(this, Y, (e) => {
      for (let i = 0; i < t(this, d).length; i++)
        t(this, d)[i].dispatch("pointerUp", e);
    });
    r(this, _, (e) => {
      if (!t(this, b).length)
        return;
      t(this, H).x = e.clientX / u.width * 2 - 1, t(this, H).y = -(e.clientY / u.height) * 2 + 1, u.camera && t(this, N).setFromCamera(t(this, H), u.camera);
      const i = [];
      for (const c of t(this, b)) {
        const m = t(this, N).intersectObject(c.target);
        m.length && (c.intersection = m[0], i.push(c));
      }
      let n = !1;
      const a = t(this, d).filter(
        (c) => !i.find((m) => m.object3D.uuid === c.object3D.uuid)
      ), E = i.sort(
        (c, m) => m.object3D.position.z - c.object3D.position.z
      ).filter((c, m) => n ? (a.push(c), !1) : (n = !c.propagation, !0)), w = E.filter(
        (c) => !t(this, d).find((m) => m.object3D.uuid === c.object3D.uuid)
      );
      for (let c = 0; c < a.length; c++)
        a[c].dispatch("pointerLeave", e);
      for (let c = 0; c < w.length; c++)
        w[c].dispatch("pointerEnter", e);
      h(this, d, E);
      for (let c = 0; c < t(this, d).length; c++)
        t(this, d)[c].dispatch("pointerMove", e);
    });
    u.containerElement.addEventListener(
      "pointerdown",
      t(this, W)
    ), u.containerElement.addEventListener("pointerup", t(this, Y)), u.containerElement.addEventListener(
      "pointermove",
      t(this, _)
    );
  }
  destroy() {
    u.containerElement.removeEventListener(
      "pointerdown",
      t(this, W)
    ), u.containerElement.removeEventListener(
      "pointerup",
      t(this, Y)
    ), u.containerElement.removeEventListener(
      "pointermove",
      t(this, _)
    ), h(this, b, []), h(this, d, []);
  }
  add(e, i) {
    if (t(this, b).find((a) => a.object3D.uuid === e.uuid))
      return;
    const n = new ze(j({
      object3D: e
    }, i));
    t(this, b).push(n);
  }
  remove(e) {
    h(this, b, t(this, b).filter(
      (i) => i.object3D.uuid !== e.uuid
    )), h(this, d, t(this, d).filter((i) => i.object3D.uuid !== e.uuid));
  }
}
b = new WeakMap(), d = new WeakMap(), H = new WeakMap(), N = new WeakMap(), W = new WeakMap(), Y = new WeakMap(), _ = new WeakMap();
var U, y, l, O, f, L, T, C, $, M, g, B, G;
class Le {
  constructor() {
    r(this, U, `https://unpkg.com/three@0.${De}.x`);
    r(this, y, null);
    r(this, l, null);
    r(this, O, null);
    r(this, f, /* @__PURE__ */ new Map());
    r(this, L, 0);
    r(this, T, 0);
    r(this, C, 0);
    r(this, $, 2);
    r(this, M, !1);
    r(this, g, null);
    r(this, B, () => {
      h(this, L, t(this, y).clientWidth), h(this, T, t(this, y).clientHeight), h(this, C, Math.min(t(this, $), devicePixelRatio || 1)), t(this, l).setPixelRatio(t(this, C)), t(this, l).setSize(t(this, L), t(this, T)), t(this, g) && (t(this, g).setPixelRatio(t(this, C)), t(this, g).setSize(t(this, L), t(this, T)));
    });
    r(this, G, () => {
      t(this, l).setRenderTarget(null), t(this, f).forEach((e) => {
        this.render(e);
      });
    });
  }
  get CDNVersion() {
    return t(this, U);
  }
  get containerElement() {
    return t(this, y);
  }
  get webglRenderer() {
    return t(this, l);
  }
  get raycaster() {
    return t(this, O);
  }
  get views() {
    return t(this, f);
  }
  get view() {
    return this.getView("default");
  }
  get camera() {
    return this.getView("default").camera;
  }
  get scene() {
    return this.getView("default").scene;
  }
  get width() {
    return t(this, L);
  }
  get height() {
    return t(this, T);
  }
  get pixelRatio() {
    return t(this, C);
  }
  get composer() {
    return t(this, g);
  }
  setup(e) {
    if (t(this, M)) {
      console.warn("[en3.setup]: You are trying to setup en3 again.");
      return;
    }
    h(this, y, ce(e == null ? void 0 : e.containerElement) || document.body), h(this, $, (e == null ? void 0 : e.maxPixelRatio) || 2), h(this, l, new Re(e == null ? void 0 : e.webGLRendererParameters)), t(this, l).domElement.style.cssText = `
      position: fixed;
      left: 0;
      top: 0;
      width: 100%;
      height: 100%;
      z-index: ${(e == null ? void 0 : e.zIndex) || 0};
    `, (t(this, y).shadowRoot || t(this, y)).append(t(this, l).domElement), t(this, f).set(
      "default",
      new ae("default", j({
        sizeElement: t(this, y)
      }, e == null ? void 0 : e.view))
    ), h(this, O, new Ve()), e != null && e.composer && (h(this, g, new e.composer(t(this, l))), this.render = () => {
      t(this, g).render();
    }), h(this, M, !0), se.subscribe(t(this, B), be.CANVAS), re.subscribe(t(this, G), { order: ye.CANVAS });
  }
  destroy() {
    if (!t(this, M)) {
      console.warn(
        "[en3.setup]: You are trying to destory en3 but it has not been initialized."
      );
      return;
    }
    se.unsubscribe(t(this, B)), re.unsubscribe(t(this, G)), t(this, O).destroy(), t(this, f).forEach((e) => {
      e.destroy();
    }), t(this, l).dispose(), t(this, l).domElement.remove(), h(this, l, null), t(this, g).dispose(), h(this, g, null), h(this, M, !1);
  }
  createView(e, i) {
    const n = t(this, f).size, a = new ae(e, i);
    return t(this, f).set(e, a), n === 1 && t(this, l).setScissorTest(!0), a;
  }
  getView(e) {
    return t(this, f).get(e);
  }
  destroyView(e) {
    const i = t(this, f).get(e);
    i && (t(this, f).delete(e), i.destroy(), t(this, f).size <= 1 && t(this, l).setScissorTest(!1));
  }
  render(e) {
    var i;
    if (t(this, f).size > 1 || this.view.isClipped) {
      const n = e.box.left + e.box.CSSTranslation.x + e.box.scrollValue.x, a = u.height - e.box.height - e.box.top + e.box.CSSTranslation.y + e.box.scrollValue.y * -1;
      t(this, l).setScissor(n, a, e.box.width, e.box.height), t(this, l).setViewport(
        n,
        a,
        e.box.width,
        e.box.height
      );
    }
    (i = e.beforeRenderCallback) == null || i.call(e), t(this, l).render(e.scene, e.camera);
  }
}
U = new WeakMap(), y = new WeakMap(), l = new WeakMap(), O = new WeakMap(), f = new WeakMap(), L = new WeakMap(), T = new WeakMap(), C = new WeakMap(), $ = new WeakMap(), M = new WeakMap(), g = new WeakMap(), B = new WeakMap(), G = new WeakMap();
const u = new Le();
function je(s, e, i) {
  const n = e.x, a = e.y, z = i || s.image.width / s.image.height;
  let E = 0, w = 0, c = s.offset.x, m = s.offset.y, oe = s.rotation, le = s.center.x, de = s.center.y;
  n / a > z ? (E = 1, w = a / n * z) : (w = 1, E = n / a / z), s.matrix.setUvTransform(c, m, E, w, oe, le, de);
}
const K = new q(), Te = new q();
function ke(s, e = "default") {
  const i = u.getView(e), { width: n, height: a } = u, z = n / a;
  s instanceof q ? K.copy(s) : K.set(...s);
  const E = i.camera.getWorldPosition(Te).distanceTo(K);
  if (i.camera instanceof X)
    return {
      width: n / i.camera.zoom,
      height: a / i.camera.zoom,
      factor: 1,
      distance: E,
      aspect: z
    };
  {
    const w = i.camera.fov * Math.PI / 180, c = 2 * Math.tan(w / 2) * E, m = c * (n / a);
    return { width: m, height: c, factor: n / m, distance: E, aspect: z };
  }
}
function Fe(s, e) {
  s.traverse((i) => {
    i instanceof Z && i.material && (Array.isArray(i.material) ? i.material : [i.material]).forEach(e);
  });
}
function Ie(s, e) {
  s.traverse((i) => {
    i instanceof Z && e(i);
  });
}
export {
  ae as En3View,
  je as coverTexture,
  ve as dispose,
  u as en3,
  ke as getCurrentViewport,
  Fe as traverseMaterials,
  Ie as traverseMeshes
};

==> aptechka-0.44.1.tgz: package/lib/en3/index.js <==
var ue = Object.defineProperty, me = Object.defineProperties;
var fe = Object.getOwnPropertyDescriptors;
var J = Object.getOwnPropertySymbols;
var ge = Object.prototype.hasOwnProperty, Ee = Object.prototype.propertyIsEnumerable;
var ee = (s) => {
  throw TypeError(s);
};
var Q = (s, e, i) => e in s ? ue(s, e, { enumerable: !0, configurable: !0, writable: !0, value: i }) : s[e] = i, j = (s, e) => {
  for (var i in e || (e = {}))
    ge.call(e, i) && Q(s, i, e[i]);
  if (J)
    for (var i of J(e))
      Ee.call(e, i) && Q(s, i, e[i]);
  return s;
}, te = (s, e) => me(s, fe(e));
var ie = (s, e, i) => e.has(s) || ee("Cannot " + i);
var t = (s, e, i) => (ie(s, e, "read from private field"), i ? i.call(s) : e.get(s)), r = (s, e, i) => e.has(s) ? ee("Cannot add the same private member more than once") : e instanceof WeakSet ? e.add(s) : e.set(s, i), h = (s, e, i, n) => (ie(s, e, "write to private field"), n ? n.call(s, i) : e.set(s, i), i);
import { RESIZE_ORDER as be, TICK_ORDER as ye } from "../order/index.js";
import { windowResizer as se } from "../window-resizer/index.js";
import { ticker as re } from "../ticker/index.js";
import { g as ce } from "../dom-CVWzyXPH.js";
import "../index-euf5anj6.js";
import { Mesh as Z, OrthographicCamera as X, PerspectiveCamera as he, Scene as xe, Vector2 as we, Raycaster as pe, REVISION as De, WebGLRenderer as Re, Vector3 as q } from "three";
import { LayoutBox as ne } from "../layout-box/index.js";
function ve(s) {
  const e = (i) => {
    i.dispose();
    for (const n of Object.keys(i)) {
      const a = i[n];
      a && typeof a == "object" && "minFilter" in a && a.dispose();
    }
  };
  s.traverse((i) => {
    if (i instanceof Z) {
      if (i.geometry.dispose(), !Array.isArray(i.material) && i.material.isMaterial)
        e(i.material);
      else if (Array.isArray(i.material))
        for (const n of i.material) e(n);
    }
  });
}
var S, o, V, p, x, P, D, R;
class ae {
  constructor(e, i) {
    r(this, S);
    r(this, o);
    r(this, V);
    r(this, p);
    r(this, x);
    r(this, P);
    r(this, D);
    r(this, R);
    h(this, S, e), h(this, o, (i == null ? void 0 : i.cameraType) === "orthographic" ? new X() : new he()), h(this, V, new xe()), h(this, p, []), h(this, x, (i == null ? void 0 : i.cameraDistance) || 1e3), t(this, o).near = (i == null ? void 0 : i.cameraNear) || 1, t(this, o).far = (i == null ? void 0 : i.cameraFar) || 11e3, h(this, P, (i == null ? void 0 : i.cameraFov) || "auto"), t(this, o).position.z = t(this, x), t(this, o).name = `Cameras.${t(this, S)}`, h(this, D, ce(i == null ? void 0 : i.sizeElement) || document.documentElement), this.beforeRenderCallback = i == null ? void 0 : i.beforeRender, h(this, R, new ne(t(this, D))), t(this, R).onResize(() => {
      this.resize();
    });
  }
  get name() {
    return t(this, S);
  }
  get camera() {
    return t(this, o);
  }
  get scene() {
    return t(this, V);
  }
  get box() {
    return t(this, R);
  }
  get cameraDistance() {
    return t(this, x);
  }
  set cameraDistance(e) {
    h(this, x, e), this.resize();
  }
  get sizeElement() {
    return t(this, D);
  }
  get isClipped() {
    return t(this, D) !== u.containerElement;
  }
  resize() {
    const { width: e, height: i } = t(this, R);
    t(this, o).userData.controlled || (t(this, o).position.z = t(this, x)), t(this, o) instanceof he ? (t(this, o).aspect = e / i, t(this, o).fov = t(this, P) === "auto" ? 2 * Math.atan(i / 2 / t(this, x)) * (180 / Math.PI) : t(this, o).userData.controlled ? t(this, o).fov : t(this, P)) : t(this, o) instanceof X && (t(this, o).left = e / -2, t(this, o).right = e / 2, t(this, o).top = i / 2, t(this, o).bottom = i / -2), t(this, o).updateProjectionMatrix();
  }
  destroy() {
    t(this, p).forEach((e) => {
      e.userData.box.destroy();
    }), t(this, V).clear(), ve(t(this, V)), u.destroyView(this.name), t(this, R).destroy();
  }
  attachToHTMLElement(e, i, n) {
    const a = new ne(e, te(j({}, n), {
      containerElement: t(this, D),
      cartesian: !0,
      scrollStep: !this.isClipped
    }));
    return a.bindObject(i), i.userData.box = a, t(this, p).push(i), i;
  }
  detachFromHTMLElement(e) {
    h(this, p, t(this, p).filter((i) => i === e ? (e.userData.box.destroy(), !1) : !0));
  }
  add(...e) {
    const i = e[0], n = e[1], a = e[2];
    return n && this.attachToHTMLElement(n, i, a), this.scene.add(i), i;
  }
  remove(e, i) {
    this.scene.remove(e), i && this.detachFromHTMLElement(e);
  }
}
S = new WeakMap(), o = new WeakMap(), V = new WeakMap(), p = new WeakMap(), x = new WeakMap(), P = new WeakMap(), D = new WeakMap(), R = new WeakMap();
var A, k, F, v, I;
class ze {
  constructor(e) {
    r(this, A);
    r(this, k);
    r(this, F);
    r(this, v);
    r(this, I);
    h(this, A, e.targetName || void 0), h(this, k, e.eventDispatcher || e.object3D), h(this, F, e.propagation || !1), h(this, v, e.object3D), h(this, I, t(this, A) ? () => t(this, v).getObjectByName(t(this, A)) || t(this, v) : () => t(this, v));
  }
  get object3D() {
    return t(this, v);
  }
  get eventDispatcher() {
    return t(this, k);
  }
  get propagation() {
    return t(this, F);
  }
  get target() {
    return t(this, I).call(this);
  }
  dispatch(e, i) {
    this.eventDispatcher.dispatchEvent(j({
      originalEvent: i,
      type: e
    }, this.intersection));
  }
}
A = new WeakMap(), k = new WeakMap(), F = new WeakMap(), v = new WeakMap(), I = new WeakMap();
var b, d, H, N, W, Y, _;
class Ve {
  constructor() {
    r(this, b, []);
    r(this, d, []);
    r(this, H, new we());
    r(this, N, new pe());
    r(this, W, (e) => {
      for (let i = 0; i < t(this, d).length; i++)
        t(this, d)[i].dispatch("pointerDown", e);
    });
    r(this, Y, (e) => {
      for (let i = 0; i < t(this, d).length; i++)
        t(this, d)[i].dispatch("pointerUp", e);
    });
    r(this, _, (e) => {
      if (!t(this, b).length)
        return;
      t(this, H).x = e.clientX / u.width * 2 - 1, t(this, H).y = -(e.clientY / u.height) * 2 + 1, u.camera && t(this, N).setFromCamera(t(this, H), u.camera);
      const i = [];
      for (const c of t(this, b)) {
        const m = t(this, N).intersectObject(c.target);
        m.length && (c.intersection = m[0], i.push(c));
      }
      let n = !1;
      const a = t(this, d).filter(
        (c) => !i.find((m) => m.object3D.uuid === c.object3D.uuid)
      ), E = i.sort(
        (c, m) => m.object3D.position.z - c.object3D.position.z
      ).filter((c, m) => n ? (a.push(c), !1) : (n = !c.propagation, !0)), w = E.filter(
        (c) => !t(this, d).find((m) => m.object3D.uuid === c.object3D.uuid)
      );
      for (let c = 0; c < a.length; c++)
        a[c].dispatch("pointerLeave", e);
      for (let c = 0; c < w.length; c++)
        w[c].dispatch("pointerEnter", e);
      h(this, d, E);
      for (let c = 0; c < t(this, d).length; c++)
        t(this, d)[c].dispatch("pointerMove", e);
    });
    u.containerElement.addEventListener(
      "pointerdown",
      t(this, W)
    ), u.containerElement.addEventListener("pointerup", t(this, Y)), u.containerElement.addEventListener(
      "pointermove",
      t(this, _)
    );
  }
  destroy() {
    u.containerElement.removeEventListener(
      "pointerdown",
      t(this, W)
    ), u.containerElement.removeEventListener(
      "pointerup",
      t(this, Y)
    ), u.containerElement.removeEventListener(
      "pointermove",
      t(this, _)
    ), h(this, b, []), h(this, d, []);
  }
  add(e, i) {
    if (t(this, b).find((a) => a.object3D.uuid === e.uuid))
      return;
    const n = new ze(j({
      object3D: e
    }, i));
    t(this, b).push(n);
  }
  remove(e) {
    h(this, b, t(this, b).filter(
      (i) => i.object3D.uuid !== e.uuid
    )), h(this, d, t(this, d).filter((i) => i.object3D.uuid !== e.uuid));
  }
}
b = new WeakMap(), d = new WeakMap(), H = new WeakMap(), N = new WeakMap(), W = new WeakMap(), Y = new WeakMap(), _ = new WeakMap();
var U, y, l, O, f, L, T, C, $, M, g, B, G;
class Le {
  constructor() {
    r(this, U, `https://unpkg.com/three@0.${De}.x`);
    r(this, y, null);
    r(this, l, null);
    r(this, O, null);
    r(this, f, /* @__PURE__ */ new Map());
    r(this, L, 0);
    r(this, T, 0);
    r(this, C, 0);
    r(this, $, 2);
    r(this, M, !1);
    r(this, g, null);
    r(this, B, () => {
      h(this, L, t(this, y).clientWidth), h(this, T, t(this, y).clientHeight), h(this, C, Math.min(t(this, $), devicePixelRatio || 1)), t(this, l).setPixelRatio(t(this, C)), t(this, l).setSize(t(this, L), t(this, T)), t(this, g) && (t(this, g).setPixelRatio(t(this, C)), t(this, g).setSize(t(this, L), t(this, T)));
    });
    r(this, G, () => {
      t(this, l).setRenderTarget(null), t(this, f).forEach((e) => {
        this.render(e);
      });
    });
  }
  get CDNVersion() {
    return t(this, U);
  }
  get containerElement() {
    return t(this, y);
  }
  get webglRenderer() {
    return t(this, l);
  }
  get raycaster() {
    return t(this, O);
  }
  get views() {
    return t(this, f);
  }
  get view() {
    return this.getView("default");
  }
  get camera() {
    return this.getView("default").camera;
  }
  get scene() {
    return this.getView("default").scene;
  }
  get width() {
    return t(this, L);
  }
  get height() {
    return t(this, T);
  }
  get pixelRatio() {
    return t(this, C);
  }
  get composer() {
    return t(this, g);
  }
  setup(e) {
    if (t(this, M)) {
      console.warn("[en3.setup]: You are trying to setup en3 again.");
      return;
    }
    h(this, y, ce(e == null ? void 0 : e.containerElement) || document.body), h(this, $, (e == null ? void 0 : e.maxPixelRatio) || 2), h(this, l, new Re(e == null ? void 0 : e.webGLRendererParameters)), t(this, l).domElement.style.cssText = `
      position: fixed;
      left: 0;
      top: 0;
      width: 100%;
      height: 100%;
      z-index: ${(e == null ? void 0 : e.zIndex) || 0};
    `, (t(this, y).shadowRoot || t(this, y)).append(t(this, l).domElement), t(this, f).set(
      "default",
      new ae("default", j({
        sizeElement: t(this, y)
      }, e == null ? void 0 : e.view))
    ), h(this, O, new Ve()), e != null && e.composer && (h(this, g, new e.composer(t(this, l))), this.render = () => {
      t(this, g).render();
    }), h(this, M, !0), se.subscribe(t(this, B), be.CANVAS), re.subscribe(t(this, G), { order: ye.CANVAS });
  }
  destroy() {
    if (!t(this, M)) {
      console.warn(
        "[en3.setup]: You are trying to destory en3 but it has not been initialized."
      );
      return;
    }
    se.unsubscribe(t(this, B)), re.unsubscribe(t(this, G)), t(this, O).destroy(), t(this, f).forEach((e) => {
      e.destroy();
    }), t(this, l).dispose(), t(this, l).domElement.remove(), h(this, l, null), t(this, g).dispose(), h(this, g, null), h(this, M, !1);
  }
  createView(e, i) {
    const n = t(this, f).size, a = new ae(e, i);
    return t(this, f).set(e, a), n === 1 && t(this, l).setScissorTest(!0), a;
  }
  getView(e) {
    return t(this, f).get(e);
  }
  destroyView(e) {
    const i = t(this, f).get(e);
    i && (t(this, f).delete(e), i.destroy(), t(this, f).size <= 1 && t(this, l).setScissorTest(!1));
  }
  render(e) {
    var i;
    if (t(this, f).size > 1 || this.view.isClipped) {
      const n = e.box.left + e.box.CSSTranslation.x + e.box.scrollValue.x, a = u.height - e.box.height - e.box.top + e.box.CSSTranslation.y + e.box.scrollValue.y * -1;
      t(this, l).setScissor(n, a, e.box.width, e.box.height), t(this, l).setViewport(
        n,
        a,
        e.box.width,
        e.box.height
      );
    }
    (i = e.beforeRenderCallback) == null || i.call(e), t(this, l).render(e.scene, e.camera);
  }
}
U = new WeakMap(), y = new WeakMap(), l = new WeakMap(), O = new WeakMap(), f = new WeakMap(), L = new WeakMap(), T = new WeakMap(), C = new WeakMap(), $ = new WeakMap(), M = new WeakMap(), g = new WeakMap(), B = new WeakMap(), G = new WeakMap();
const u = new Le();
function ke(s, e, i) {
  const n = e.x, a = e.y, z = i || s.image.width / s.image.height;
  let E = 0, w = 0, c = s.offset.x, m = s.offset.y, oe = s.rotation, le = s.center.x, de = s.center.y;
  n / a > z ? (E = 1, w = a / n * z) : (w = 1, E = n / a / z), s.matrix.setUvTransform(c, m, E, w, oe, le, de);
}
const K = new q(), Te = new q();
function Fe(s, e = "default") {
  const i = u.getView(e), { width: n, height: a } = u, z = n / a;
  s instanceof q ? K.copy(s) : K.set(...s);
  const E = i.camera.getWorldPosition(Te).distanceTo(K);
  if (i.camera instanceof X)
    return {
      width: n / i.camera.zoom,
      height: a / i.camera.zoom,
      factor: 1,
      distance: E,
      aspect: z
    };
  {
    const w = i.camera.fov * Math.PI / 180, c = 2 * Math.tan(w / 2) * E, m = c * (n / a);
    return { width: m, height: c, factor: n / m, distance: E, aspect: z };
  }
}
function Ie(s, e) {
  s.traverse((i) => {
    i instanceof Z && i.material && (Array.isArray(i.material) ? i.material : [i.material]).forEach(e);
  });
}
function Ne(s, e) {
  s.traverse((i) => {
    i instanceof Z && e(i);
  });
}
export {
  ae as En3View,
  ke as coverTexture,
  ve as dispose,
  u as en3,
  Fe as getCurrentViewport,
  Ie as traverseMaterials,
  Ne as traverseMeshes
};
